/*********************************************************************
* FileName:        MailBox.h
********************************************************************/

// Mailbox states
#define MBS_Type_Mask					0x03
#define MBS_Type_Other					0x00
#define MBS_Type_Request_or_Response	0x01
#define MBS_Type_Indication				0x02
#define MBS_Type_Confirm				0x03

typedef enum _MAILBOX_PRIMITIVE
{
	MBS_Idle						= 0x00,	
	MBS_Wait						= 0x04,
	MBS_Error						= 0x08,

	MBS_Join_Request				= 0x01,		// Requests Mailbox joins/rejoin/form a network
	MBS_Join_Confirm				= 0x03,		// Confirms if Mailbox joins/rejoin/form is complete

	MBS_Permit_Join_Request			= 0x05,		// Sets permit join state	
	MBS_Permit_Join_Indication		= 0x06,		// Indicates a device has joined, no response required
	MBS_Permit_Join_Confirm			= 0x07,		// Confirms permit join request

	MBS_Leave_Indication			= 0x0A,		// This device was instructed to leave the network

	MBS_Device_Sleep_Request		= 0x0D,		// Requests that ZigBee stack is suspended for sleep
	MBS_Device_Sleep_Confirm		= 0x0F,		// Confirms if device may now sleep

	MBS_Device_Wake_Request			= 0x11,		// Requests that ZigBee stack is resumed from sleep
	MBS_Device_Wake_Confirm			= 0x13,		// Confirms if application may now resume 

	MBS_Device_Discovery_Request	= 0x15,		// Requests that ZigBee stack polls network for other devices with a given cluster ID
	MBS_Device_Discovery_Indication	= 0x16,		// Confirms that device discovery is complete
	MBS_Device_Discovery_Confirm	= 0x17,		// A device responded with to a discovery request

	MBS_Present_Request				= 0x19,		// Requests other devices are told I exist
	MBS_Present_Indication			= 0x1A,		// Confirms MBS_Presence_Request
	MBS_Present_Confirm				= 0x1B,		// A device announced it existed

	MBS_Network_Sleep_Request		= 0x1D,		// Requests other devices go to sleep
	MBS_Network_Sleep_Indication	= 0x1E,		// Confirms MBS_Presence_Request
	MBS_Network_Sleep_Confirm		= 0x1F,		// A device announced it existed
	
	MBS_Network_Sleep_Response		= 0x21,		// Either going to sleep or waking up

	MBS_Service_Discovery_Request	= 0x25,		// Requests info from another device
	MBS_Service_Discovery_Confirm	= 0x27,		// Returns info from another device

	MBS_Custom_Request				= 0x29,		// Custom requests
	MBS_Custom_Indication			= 0x2A,		// Confirms MBS_Custom_Request
	MBS_Custom_Confirm				= 0x2B,		// An Custom frame was received

	MBS_Data_Request				= 0x2D,		// Data requests
	MBS_Data_Indication				= 0x2E,		// Confirms MBS_Data_Request
	MBS_Data_Confirm				= 0x2F,		// An Data frame was received

	MBS_Redirect_Request			= 0x31,		// Redirect requests
	MBS_Redirect_Indication			= 0x32,		// Confirms MBS_Redirect_Request
	MBS_Redirect_Confirm			= 0x33,		// An Redirect frame was received
	
	MBS_Sync_Loss_Indication		= 0x36		// lost contact with parent during poll

} MAILBOX_PRIMITIVE;

// Data confirm & indication Status values
#define DATA_STATUS_SUCCESS			0x00
#define DATA_STATUS_UNKNOWN			0x01
#define DATA_STATUS_SEQUENCE_ERROR	0x02
#define DATA_STATUS_NOT_PERMITTED	0x03
#define DATA_STATUS_TIMED_OUT		0x04
#define DATA_STATUS_RETRY_LATER		0x05
#define DATA_STATUS_STACK_FAIL		0x06
#define DATA_STATUS_FRAMES_LOST		0x07
#define DATA_STATUS_LATE_FRAME		0x08
#define DATA_STATUS_RESET_MISMATCH	0x09
#define DATA_STATUS_CHECKSUM_FAIL	0x0A
#define DATA_STATUS_DUPLICATE		0x0B  		// Internal state only, not reported in indication or confirm


/*********************************************************************/

// Return codes 

#define MBP_Success					0x00

#define ERR_STACK_NOT_IDLE  		0x01		// Stack was not idle when a _Request was issued
#define ERR_MEMORY_ERROR			0x02		// ran out of heap memory 
#define ERR_JOIN_NOT_COMPLETE		0x03		// operation not permitted, network hasn't been joined / rejoined / formed yet
#define ERR_PROG( x )				(0x80 + x ) // Internal error if > 0x80 - contect FlexiPanel quoting error number

/*********************************************************************/
// variable definitions 

#define NUM_OPTIONAL_CLUSTERS				7
#define ROM  rom
typedef unsigned char BYTE;                 // 8-bit
typedef unsigned short int WORD;            // 16-bit
typedef unsigned short long THREEBYTE;      // 24-bit
typedef unsigned long DWORD;                // 32-bit


typedef struct _LONG_ADDR
{
    BYTE v[8];
} LONG_ADDR;

typedef union _SHORT_ADDR
{
    struct _SHORT_ADDR_bits
    {
        BYTE LSB;
        BYTE MSB;
    } byte;
    WORD Val;
    BYTE v[2];
} SHORT_ADDR;

typedef union _WORD_VAL
{
    BYTE v[2];
    WORD Val;
    struct
    {
        BYTE LSB;
        BYTE MSB;
    } byte;
} WORD_VAL;

typedef union _THREEBYTE_VAL
{
    BYTE v[3];
    THREEBYTE Val;
    struct
    {
        BYTE LSB;
        BYTE MidB;
        BYTE MSB;
    } byte;
} THREEBYTE_VAL;

#define LSB(a)          ((a).v[0])
#define MSB(a)          ((a).v[1])

typedef union _REDIR_VAL
{
    BYTE v[3];
    THREEBYTE Val;
    struct
    {
	    struct _rdrFlags
	    {
	        BYTE        IsBroadcast  : 1;
	        BYTE        IsAcknowledge  : 1;
		} RedirFlags;
        WORD_VAL AddrOrClust;
    } fields;
} REDIR_VAL;

typedef union _TICK
{
    DWORD Val;
    struct _TICK_bytes
    {
        BYTE b0;
        BYTE b1;
        BYTE b2;
        BYTE b3;
    } byte;
    BYTE v[4];
} TICK;

typedef union _DWORD_VAL
{
    DWORD Val;
    struct
    {
        BYTE LOLSB;
        BYTE LOMSB;
        BYTE HILSB;
        BYTE HIMSB;
    } byte;
    struct
    {
        WORD LSW;
        WORD MSW;
    } word;
    BYTE v[4];
} DWORD_VAL;


typedef union _MAILBOX_PARAMS
{
	// States with no parameters:
	// MBS_Idle 
	// MBS_Wait 
	// MBS_Leave_Indication
	// MBS_Device_Sleep_Request
	// MBS_Device_Wake_Request
	// MBS_Redirect_Indication
 
    struct _MBS_Error
    {
        BYTE    ErrorNo;
    } MBS_Error;

    struct _MBS_Join_Request
    {
	    struct          _jrFlags
	    {
	        BYTE        Erase  : 1;
	    }               Flags;
    } MBS_Join_Request;

    struct _MBS_Join_Confirm
    {
        BYTE    	Status;
        SHORT_ADDR  ShortAddress;         
   } MBS_Join_Confirm;

    struct _MBS_Permit_Join_Request  // overlays _NLME_PERMIT_JOINING_request
    {
      BYTE	PermitDuration;
    } MBS_Permit_Join_Request;
    
    struct _MBS_Permit_Join_Confirm  // overlays _NLME_PERMIT_JOINING_confirm
    {
        BYTE    Status;
    } MBS_Permit_Join_Confirm;
    
    struct _MBS_Permit_Join_Indication  // overlays NLME_JOIN_indication exactly
    {
        BYTE                        filler1[8];
        SHORT_ADDR                  ShortAddress;
        BYTE                        filler2[6];
        LONG_ADDR                   ExtendedAddress;            
    } MBS_Permit_Join_Indication;

    struct _MBS_Device_Sleep_Confirm  
    {
        BYTE    Status;
    } MBS_Device_Sleep_Confirm;
    
    struct _MBS_Device_Wake_Confirm
    {
        BYTE    Status;   		  	// Must overlay NLME_SYNC_confirm status
    } MBS_Device_Wake_Confirm;
    
    struct _MBS_Device_Discovery_Request
    {
        WORD_VAL    FuncID;   		  
    } MBS_Device_Discovery_Request;
    
    struct _MBS_Device_Discovery_Indication  
    {
        WORD_VAL    SourceAddr;
    } MBS_Device_Discovery_Indication;
   
    struct _MBS_Device_Discovery_Confirm
    {
       BYTE    Status;   		  	
    } MBS_Device_Discovery_Confirm;
    
    struct _MBS_Service_Discovery_Request
    {
        WORD_VAL    Destination;   		  
        WORD_VAL    InfoType;   		  
        BYTE 		DataPayloadLen;	  
    } MBS_Service_Discovery_Request;
    
    struct _MBS_Service_Discovery_Confirm_AppId
    {
       	BYTE    	Status;   		  	
        WORD_VAL    InfoType;   
       	DWORD_VAL   ApplicationID;   
    } MBS_Service_Discovery_Confirm_AppId;
    
    struct _MBS_Service_Discovery_Confirm_ZDO
    {
       	BYTE    	Status;   		  	
        WORD_VAL    InfoType;   
        BYTE 		DataPayloadLen;	 
        BYTE* 		pRxData; 
    } MBS_Service_Discovery_Confirm_ZDO;
    
    struct _MBS_Present_Request
    {
	    struct      _prFlags
	    {
	        BYTE    Broadcast  : 1;
	    }           Flags;
        WORD_VAL    Destination;   		  
   } MBS_Present_Request;
    
    struct _MBS_Present_Indication  
    {
        WORD_VAL    SourceAddr;
        BYTE	    NumCluster;
        WORD_VAL    ClusterList[NUM_OPTIONAL_CLUSTERS];
    } MBS_Present_Indication;
   
    struct _MBS_Present_Confirm
    {
       BYTE    Status;   		  	
    } MBS_Present_Confirm;
    
    struct _MBS_Redirect_Request
    {
	    struct      _rrFlags
	    {
	        BYTE    Broadcast  : 1;
	    } Flags;
        WORD_VAL    Destination;   		  
		REDIR_VAL	RedirectAddr;
   } MBS_Redirect_Request;
    
    struct _MBS_Redirect_Confirm
    {
       BYTE    Status;   		  	
    } MBS_Redirect_Confirm;
    
    struct _MBS_Network_Sleep_Request
    {
	    struct          _nsrFlags
	    {
	        BYTE        Broadcast  : 1;
	    }               Flags;
        WORD_VAL 		Destination;   		  
	    WORD_VAL		ClosedownPeriod;
	    THREEBYTE_VAL	SleepPeriod;
	    WORD_VAL		WakeupPeriod;
   } MBS_Network_Sleep_Request;
    
    struct _MBS_Network_Sleep_Indication  
    {
        WORD_VAL    	SourceAddr;
	    WORD_VAL		ClosedownPeriod;
	    THREEBYTE_VAL	SleepPeriod;
	    WORD_VAL		WakeupPeriod;
    } MBS_Network_Sleep_Indication;
   
    struct _MBS_Network_Sleep_Confirm
    {
       BYTE    Status;   		  	
    } MBS_Network_Sleep_Confirm;
    
    struct _MBS_Network_Sleep_Response
    {
	    struct _nspFlags
	    {
	        unsigned int Sleep :1;
	    } Flags;
    } MBS_Network_Sleep_Response;
   
     struct _MBS_Custom_Request
    {
	    BYTE			Custom_FID;
	    struct _crFlags
	    {
	        BYTE        Broadcast  : 1;
	    } Flags;
        WORD_VAL 		Destination;   		  
        BYTE 			DataPayloadLen;	  
   } MBS_Custom_Request;
    
    struct _MBS_Custom_Indication  
    {
	    BYTE			Custom_FID;
        WORD_VAL    	SourceAddr;
        BYTE 			DataPayloadLen;	  
        BYTE* pRxData; 
    } MBS_Custom_Indication;
   
    struct _MBS_Custom_Confirm
    {
       BYTE    Status;   		  	
    } MBS_Custom_Confirm;
    
     struct _MBS_Data_Request
    {
	    struct _drFlags
	    {
	        BYTE        IsBroadcast  : 1;
	        BYTE        IsAcknowledge  : 1;
	        BYTE        reserved  : 1;
	        BYTE        IsRetry  : 1;
		} Flags;
        WORD_VAL 		Destination;   	
        BYTE 			DataPayloadLen;	  
    } MBS_Data_Request;
    
    struct _MBS_Data_Indication  
    {
       	BYTE   	 		Status;   		  	
        WORD_VAL    	SourceAddr;
       	BYTE    		Seq;   		  	
        BYTE 			DataPayloadLen;	  
        BYTE* 			pRxData; 
    } MBS_Data_Indication;
   
    struct _MBS_Data_Confirm
    {
       BYTE    		Status;   		  	
       BYTE    		Seq;   		  	
       WORD_VAL    	RetryDelay;   		  	
    } MBS_Data_Confirm;
    
    
} MAILBOX_PARAMS;

// The following line overlays the MailBox parameters with the ZigBee parameters.  This only works
// if no mailbox parameter structure is larger than the largest ZigBee parameter!
typedef union _PARAMS	{BYTE Status;} PARAMS;
extern PARAMS           params;
#define pMailBoxParam ( (MAILBOX_PARAMS*) (&params))
extern BYTE MailBoxState;	// Current state of Mailbox state machine


/*********************************************************************/

// Macros
#ifndef FRAME_BEACON	// If zigbeetasks.h not included
typedef enum _ZIGBEE_PRIMITIVE  {NO_PRIMITIVE=0} ZIGBEE_PRIMITIVE;
extern ZIGBEE_PRIMITIVE    	currentPrimitive;
#endif

#ifdef I_AM_RFD
	#define MbxIsZigBeeReady()		(ZigBeeReady() && ZigBeeStatus.flags.bits.bDataRequestComplete)
#else
	#define MbxIsZigBeeReady()		( ZigBeeReady() )
#endif
#define IsStackIdle()		(currentPrimitive == NO_PRIMITIVE && MbxIsZigBeeReady())
#define RANDOM_LSB                      (TMR0L)
#define RANDOM_MSB                      (TMR0H)

// Buffer locations
#define pTxZDO  	(&TxBuffer[TX_DATA_START+3])
#define pTxCustom  	(&TxBuffer[TX_DATA_START+8])
#define pTxData  	(&TxBuffer[TX_DATA_START+10])
extern BYTE             TxBuffer[];
#define TX_DATA_START   0

#define MAILBOX_REDIRECT_CLUSTER_1BYTE			0xFE	// Cluster used for redirection by remote device
#define MAILBOX_NULL_CLUSTER_1BYTE				0xFF	// Unused cluster number used as filler
#define MAILBOX_REDIRECT_CLUSTER_2BYTE			0xFFFE	// Cluster used for redirection by remote device
#define MAILBOX_NULL_CLUSTER_2BYTE				0xFFFF	// Unused cluster number used as filler

/*********************************************************************/

// API function calls

void MailBoxTasks( void );
void MailBoxInit( void );

/*********************************************************************/

// API function callbacks

void ZigBeeHook( void );
void PriorityUserInterruptHandler( void );
 
/*********************************************************************/

// MIB attributes 

// Except for MIB_RetryLater and MIB_RedirectAddr, these should only be modified before a network has been joined for the first time

extern ROM DWORD_VAL 	MIB_AppID;
extern ROM BYTE 		MIB_SeqBufSiz;
extern ROM BYTE 		MIB_PALevel;  
extern ROM LONG_ADDR    macLongAddr;
//extern volatile ROM NODE_SIMPLE_DESCRIPTOR Config_Simple_Descriptors[];  // I need to define this better  ????
extern ROM DWORD 		MIB_MailBoxTimeout;
extern ROM DWORD 		MIB_PersistTime;
extern ROM DWORD 		MIB_PollRate;  		

#define GetMACAddress( x )              NVMRead( x, (ROM void*)&macLongAddr, sizeof(LONG_ADDR) )
#define GetMACAddressByte( y, x )       NVMRead( x, (ROM void*)((int)&macLongAddr + (int)y), 1 )
#define PutMACAddress( x )              NVMWrite((NVM_ADDR*)&macLongAddr, (BYTE*)x, sizeof(LONG_ADDR))

#define GetPALevel( x )             	NVMRead( x, (ROM void*)&MIB_PALevel, sizeof(MIB_PALevel) )
#define PutPALevel( x )              	NVMWrite((NVM_ADDR*)&MIB_PALevel, (BYTE*)x, sizeof(MIB_PALevel))

#define GetAppID( x )             	 	NVMRead( x, (ROM void*)&MIB_AppID, sizeof(MIB_AppID) )
#define PutAppID( x )              		NVMWrite((NVM_ADDR*)&MIB_AppID, (BYTE*)x, sizeof(MIB_AppID))

#define GetSeqBufSiz( x )             	NVMRead( x, (ROM void*)&MIB_SeqBufSiz, sizeof(MIB_SeqBufSiz) )
#define PutSeqBufSiz( x )              	NVMWrite((NVM_ADDR*)&MIB_SeqBufSiz, (BYTE*)x, sizeof(MIB_SeqBufSiz))

#define GetMailBoxTimeout( x )          NVMRead( x, (ROM void*)&MIB_MailBoxTimeout, sizeof(MIB_MailBoxTimeout) )
#define PutMailBoxTimeout( x )          NVMWrite((NVM_ADDR*)&MIB_MailBoxTimeout, (BYTE*)x, sizeof(MIB_MailBoxTimeout))

#define GetPersistTime( x )             NVMRead( x, (ROM void*)&MIB_PersistTime, sizeof(MIB_PersistTime) )
#define PutPersistTime( x )             NVMWrite((NVM_ADDR*)&MIB_PersistTime, (BYTE*)x, sizeof(MIB_PersistTime))

// MIB_RetryDelay is in RAM, you can set/read it directly
extern WORD_VAL 		MIB_RetryDelay;

// MIB_RedirectAddr is in RAM, you can set/read it directly
extern REDIR_VAL 		MIB_RedirectAddr;

// ApplicationEP & Cluster List P are just a bit too complicated to define, so they are actual functions
void GetAppEP( void* EPVal );
void PutAppEP( void* EPVal );

// Up to 7 (i = 1 to 7) functional clusters may be supported in addition to the mandatory cluster 0x0000 (i = 0)
// Cluster 0xFFFF is a null cluster number used to fill up the list
// with version 1.0, only clusters 0x0001 to 0x00FE are available. 
void GetClusterList( void* ClusterList );
void PutClusterList( void* ClusterList );
	

/*********************************************************************/

// Utility functions

// nonvolatile memory read and write
// memory is checked before writing, so repeated writinf the same value does not exhaust flash
typedef ROM BYTE NVM_ADDR;
void NVMWrite(NVM_ADDR *dest, BYTE *src, BYTE count);
#define NVMRead(dest, src, count)   memcpypgm2ram(dest, src, count)
 
// 4-byte timer in 62500ths of a second (i.e. rolls over after 19 hours)
TICK TickGet(void);
#define TickGetDiff(a,b) (a.Val - b.Val)

// dynamic memory.  maximum allocation 126 bytes
unsigned char * SRAMalloc(unsigned char nBytes);  //  returns 0 if can't allocate memory
void SRAMfree(unsigned char * pSRAM);

 
/*********************************************************************/
