//************************************************************************
// FileName:        Pixie DARC.c
//                  Sample Mailbox application - Data Acquisition and Remote control
//
//*************************************************************************

// Basic process in this application:
// 1. If I'm started up with the join button down, erase network data
// 2. If I'm not joined to anything, attempt to join
// 3. If join fails, or any errors occur, then reset
// 4. If join succeeds, send present announcement 
// 5. If join button pressed, permit joining (routers & coordinator only)
// 6. On receipt of instruction from another device, process and respond to that device

//*************************************************************************

// Compile switches - you can eeither uncomment here or in the build settings
//#define DARC_COORDINATOR
//#define DARC_ROUTER
//#define DARC_FAST_END
//#define DARC_SLEEPY_END

//#define PIXIE
//#define PIXIE_LITE

#if (!defined(PIXIE) && !defined(PIXIE_LITE))
#error "PIXIE_xxx must be defined"
#endif

#if (!defined(DARC_COORDINATOR) && !defined(DARC_ROUTER) && !defined(DARC_FAST_END) && !defined(DARC_SLEEPY_END))
#error "DARC_xxx must be defined"
#endif


//*************************************************************************

// includes

#include <p18cxxx.h>   

// MailBox definitions
#include "MailBox.h"
#include <delays.h> 		// for delay functions
#include <string.h>         // for memcpy-type functions

//*************************************************************************

// DARC commands
#define SET_IO_VAL		0x01
#define GET_IO_VAL		0x02
#define GET_AN_VAL		0x03
#define SET_AN_CHANS	0x04
#define COMMAND_STREAM	0x05

#define IO_RA0	0x00
#define IO_RA1	0x01 
#define IO_RA2	0x02
#define IO_RA3	0x03
#define IO_RB4	0x14
#define IO_RB5	0x15
#define IO_RB6	0x16
#define IO_RC6	0x26
#define IO_RC7	0x27
#define IO_RE0	0x40
#define IO_RE1	0x41
#define IO_RE2	0x42
#define IO_AN0	0x80
#define IO_AN1	0x81
#define IO_AN2	0x82
#define IO_AN3	0x83
#define IO_AN5	0xC0
#define IO_AN6	0xC1
#define IO_AN7	0xC2

#define DARC_STATUS_SUCCESS			 	0x00
#define DARC_SYNTAX_ERROR 				0x01
#define DARC_STATUS_OUT_OF_RANGE	 	0x02
#define DARC_STATUS_IO_NOT_POSSIBLE		0x03

void SetIOVal( BYTE IOval );
void GetIOVal( BYTE IOval );
void GetANVal( BYTE IOval );
void SetANChannels( void );
void SetStream( void );

BYTE NumANchannels = 0;
BYTE StreamChannel = 0;
BYTE StreamRate = 0;
#ifdef DARC_SLEEPY_END
BYTE StreamCountDown ;
#else
TICK LastStream;
#endif

//********************************************************************************/
// semaphores

typedef union _DARC_FLAGS
{
    WORD Val;
    struct _Bits
    { 
		BYTE ParseError : 1;
		BYTE Joined : 1;
		BYTE PresenceAnnounced : 1;
		BYTE GoToSleep : 1;
		BYTE Erase : 1;
		BYTE ProcessInboundMessage : 1;
		BYTE SendStreamValue : 1;
		BYTE StayAwakeToStream : 1;
		BYTE Announce: 1;
	} Bits;
}  DARC_FLAGS;

DARC_FLAGS DARCFlags;  

//--------------------------------------------------------------------------------------------------------

// Cache for incoming data
#define 	MAX_CMD_LEN 8
BYTE 		MessageCache[ MAX_CMD_LEN ];
BYTE		MessageLen;
WORD_VAL	ReplyAddress;

//--------------------------------------------------------------------------------------------------------

// Nonvolatile variables.  Note that writing nonvolatile memory requires erasing a block
// of memory 0x40 bytes long, so we will ensure sure the rest of the block is not program
// memory by declaring filler before and after.  This is very ineccifient, there are better ways to do it

rom BYTE DARCfiller1[0x40];

// Check MAC address is set.  
rom BYTE nvMACset = 0;  			// Nonzero if MAC address has been set

rom BYTE DARCfiller2[0x40];

//--------------------------------------------------------------------------------------------------------

// UART is used for setting MAC address

#define CLOCK_FREQ 16000000
#define BAUD_RATE 19200
#define USART_USE_BRGH_HIGH
#if defined(USART_USE_BRGH_LOW)
    #define SPBRG_VAL   ( ((CLOCK_FREQ/BAUD_RATE)/64) - 1)
#else
    #define SPBRG_VAL   ( ((CLOCK_FREQ/BAUD_RATE)/16) - 1)
#endif

#if SPBRG_VAL > 255
    #error "Calculated SPBRG value is out of range for currnet CLOCK_FREQ."
#endif

#if defined(USART_USE_BRGH_HIGH)
#define TXSTA_VAL 0x24
#else
#define TXSTA_VAL 0x20
#endif

//--------------------------------------------------------------------------------

// I/O pin definitions

#define STATUS_LED					PORTAbits.RA4
#define TRIS_STATUS_LED				TRISAbits.TRISA4

#if !defined(DEBUG_MODE)
	#define BIND_SWITCH             PORTBbits.RB7			// Bind = RB7 in normal operation
	#define TRIS_BIND_SWITCH        TRISBbits.TRISB7
#else
	#define BIND_SWITCH             PORTBbits.RB5			// Bind = RB5 in debug
	#define TRIS_BIND_SWITCH        TRISBbits.TRISB5
#endif

//********************************************************************************/

void PutROMString(ROM char* str);
void NetworkFailed( void );

// main program

void main(void) 
{
	// initialize I/O. 
	DARCFlags.Val = 0x0000;  
    ADCON1 = 0x0F; 		 	// Make PORTA digital I/O.
	TRIS_BIND_SWITCH = 1;	// Bind switch is input
    TRIS_STATUS_LED = 0;	// Status LED is output
    STATUS_LED = 1;			// Status LED is on until join is successful

	// Test to see if Join was presed on power-up, indicating a request for erase
    if (!BIND_SWITCH) 		
    	DARCFlags.Bits.Erase = 1;

	// set up internal oscillator for 16MHz operation if required
#ifdef OSC_INTERNAL
	OSCCONbits.IRCF1 = 1;  	// assuming <IDCF2:IDCF0>=100 on startup, changes to <IDCF2:IDCF0>=110 = 4MHz
	OSCTUNEbits.PLLEN = 1;  // PLL 4MHz -> 16MHz
	Delay1KTCYx( 100 ); 	// allow 25ms for clock to settle
#endif

	// enable watchdog timer 
    ClrWdt(); 
    WDTCONbits.SWDTEN = 1;
    
    // Initialize mailbox
    MailBoxInit();
   
   // If MAC not set, prompt and ask for it now
    if (!nvMACset)
    {
	    BYTE MACAddr[8];
	    BYTE* pMACAddr = &MACAddr[8];
		BYTE cTemp;

		// set up uart
	    TXSTA = TXSTA_VAL;
	    RCSTA = 0x90; 
    	SPBRG = SPBRG_VAL;

		// ask for MAC address in hex and parse it.  Note there is not very much error correction
		PutROMString( (rom char *) "MAC->" );
		do {
			pMACAddr--;
			while (PIR1bits.RCIF==0) {ClrWdt();}
			cTemp = RCREG - '0';
			if (cTemp > 0x09) cTemp -= 0x07; // A-F
			if (cTemp > 0x0F) cTemp -= 0x20; // a-f
			*pMACAddr = cTemp << 4;
			while (PIR1bits.RCIF==0) {ClrWdt();}
			cTemp = RCREG - '0';
			if (cTemp > 0x09) cTemp -= 0x07; // A-F
			if (cTemp > 0x0F) cTemp -= 0x20; // a-f
			*pMACAddr += cTemp ;
		}
		while (pMACAddr > MACAddr);
		
		// Write nonvolatile data
		PutMACAddress( pMACAddr );  
		cTemp = 1;            
		NVMWrite(&nvMACset, &cTemp, 1);
		
		// Confirm and reset
		PutROMString( (rom char *) "\r\nOK\r\n" );
		Reset();
	}
 
    while (1)
    { 
		// perform mailbox tasks
	    ClrWdt(); 
        MailBoxTasks( );
        
		// look at all possible return states and decide what to do
		switch (MailBoxState)
		{
			// Some confirms return with an error condition which should be checked for error
			case MBS_Data_Confirm:
				if (pMailBoxParam->MBS_Data_Confirm.Status != 0) NetworkFailed();
#ifdef DARC_SLEEPY_END
				if (DARCFlags.Bits.StayAwakeToStream) DARCFlags.Bits.StayAwakeToStream = 0;
#endif
				break;				 

			case MBS_Permit_Join_Confirm:
				// check join was success
				if (pMailBoxParam->MBS_Data_Confirm.Status != 0) NetworkFailed();
				break;
				
			// If an error happens, reset and rejoin
			case MBS_Leave_Indication:
			case MBS_Error:
			case MBS_Sync_Loss_Indication:
				NumANchannels = pMailBoxParam->MBS_Error.ErrorNo;
				NetworkFailed();  		// does not return

			case MBS_Join_Confirm:
				// check join was success
				if (pMailBoxParam->MBS_Data_Confirm.Status != 0) NetworkFailed();
				
				// Join succeeded
				DARCFlags.Bits.Joined = 1;
				DARCFlags.Bits.Announce = 1;
				break;

			case MBS_Present_Confirm:
				// check was success
				if (pMailBoxParam->MBS_Data_Confirm.Status != 0) NetworkFailed();
				DARCFlags.Bits.PresenceAnnounced = 1;
  				STATUS_LED = 0;			//	Initialization is complete
  				break;				

			case MBS_Permit_Join_Indication:
				// a device joined, clear the LED to indicate it.  
				if (pMailBoxParam->MBS_Data_Confirm.Status != 0) NetworkFailed();
				STATUS_LED = 0;
				break;

			case MBS_Data_Indication:
				// Data is acceptable unless we have received a more recent packet or
				// it came with a serious health warning		
				if ( pMailBoxParam->MBS_Data_Indication.Status!=DATA_STATUS_SEQUENCE_ERROR &&
					 pMailBoxParam->MBS_Data_Indication.Status!=DATA_STATUS_LATE_FRAME &&
					 pMailBoxParam->MBS_Data_Indication.DataPayloadLen <= MAX_CMD_LEN )
					 {
						// we received a message so remember it and who it was from 
						ReplyAddress = pMailBoxParam->MBS_Data_Indication.SourceAddr;
						MessageLen = pMailBoxParam->MBS_Data_Indication.DataPayloadLen;
						memcpy( MessageCache, (void*) pMailBoxParam->MBS_Data_Indication.pRxData, MessageLen );
						DARCFlags.Bits.ProcessInboundMessage = 1;
					}
				break;

#ifdef DARC_SLEEPY_END
			case MBS_Device_Sleep_Confirm:
				// Mailbox is happy to sleep
				if (pMailBoxParam->MBS_Data_Confirm.Status == 0) 
				{
					DARCFlags.Bits.GoToSleep = 1;
				}
				break;
#endif

			// no action required for other exit conditions
			default: break;
		}
		
		// Pump the mailbox tasks until idle
		if (MailBoxState!=MBS_Idle)  continue;
		
		// Any application tasks that do need to modify MailBoxState from here on

		// If I haven't joined yet, then join
		if (!DARCFlags.Bits.Joined) 
		{
			pMailBoxParam->MBS_Join_Request.Flags.Erase = ( DARCFlags.Bits.Erase ? 0x01 : 0x00 );
			MailBoxState = MBS_Join_Request;
			continue;
		}

		// If I've joined but not announced presence, do so now
		if (DARCFlags.Bits.Announce) 
		{
			DARCFlags.Bits.Announce = 0; 
			MailBoxState = MBS_Present_Request;
			pMailBoxParam->MBS_Present_Request.Flags.Broadcast = 1;
			pMailBoxParam->MBS_Present_Request.Destination.Val = 0x0000;  // Broadcast to all nodes
			continue;
		}

#ifdef DARC_SLEEPY_END
		// If ready to sleep, then sleep
		if (DARCFlags.Bits.GoToSleep) 
		{
			DARCFlags.Bits.GoToSleep = 0;

			// Perform any pre-sleep shutdown here - set to wake up on watchdog timeout
    		ClrWdt(); 

			// sleep
			Sleep();
			Nop();
			
			// Perform any post-wake operations - clear watchdog timer 
    		ClrWdt(); 

			// restart MailBox stack									
			MailBoxState = MBS_Device_Wake_Request;
			   	
			// stream data if needed
			if (StreamRate)
			{
				StreamCountDown--;
				if (!StreamCountDown)
				{
					DARCFlags.Bits.SendStreamValue = 1;
					StreamCountDown = StreamRate;
				}
			}
			continue;
		}
#endif

		// If permit join pressed, permit joining
#if (defined(DARC_COORDINATOR) || defined(DARC_COORDINATOR))
		// Note, this command should only be used for routers and coordinator only
		if (STATUS_LED==0 && BIND_SWITCH==0)  // If LED does not indicate already permitting joining and switch is on (active low) 
		{
			// permit join indefinitely; no cancel permit join 
			// code is provided if it needs to be cancelled, reset
			pMailBoxParam->MBS_Permit_Join_Request.PermitDuration = 0xFF;  
			MailBoxState = MBS_Permit_Join_Request;
			STATUS_LED = 1;
			continue;
		}
#endif

		// process a command, if one received
		if (DARCFlags.Bits.ProcessInboundMessage)
		{
			DARCFlags.Bits.ProcessInboundMessage = 0;
			
			// set up default response
			pTxData[0] = DARC_SYNTAX_ERROR;
			pMailBoxParam->MBS_Data_Request.DataPayloadLen = 0x01;
			
			switch (MessageCache[0])
			{
				case SET_IO_VAL:  		
					if (MessageLen==0x03)	SetIOVal(MessageCache[1]); 
					break;
				case GET_IO_VAL:  	
					if (MessageLen==0x02)	GetIOVal(MessageCache[1]); 
					break;
				case GET_AN_VAL:  		
					if (MessageLen==0x02)	GetANVal(MessageCache[1]); 
					break;
				case SET_AN_CHANS: 		
					if (MessageLen==0x02)	SetANChannels(); 
					break;
				case COMMAND_STREAM:  	
					if (MessageLen==0x03)	SetStream();
					break;				
			}
			
			// compile reply
			pMailBoxParam->MBS_Data_Request.Flags.IsBroadcast = 0;  
			pMailBoxParam->MBS_Data_Request.Flags.IsAcknowledge = 0;  
			pMailBoxParam->MBS_Data_Request.Flags.IsRetry = 0;  
			pMailBoxParam->MBS_Data_Request.Destination.Val = ReplyAddress.Val;
			MailBoxState = MBS_Data_Request;
			continue;
		}


	// Streaming, for non-sleepy devices 
#ifndef DARC_SLEEPY_END
	if (StreamRate)
	{
		TICK tNow = TickGet();
		if (TickGetDiff( tNow, LastStream ) > (((DWORD) StreamRate) << 16))
		{
			LastStream = tNow;
			DARCFlags.Bits.SendStreamValue = 1;
		}
	}
#endif

	if (DARCFlags.Bits.SendStreamValue)
	{
		DARCFlags.Bits.SendStreamValue = 0;
		DARCFlags.Bits.StayAwakeToStream = 1;
		
		// compile reply
		if (StreamChannel>=IO_AN0)
			GetANVal(StreamChannel);
		else
			GetIOVal(StreamChannel);
		pMailBoxParam->MBS_Data_Request.Flags.IsBroadcast = 0;  
		pMailBoxParam->MBS_Data_Request.Flags.IsAcknowledge = 0;  
		pMailBoxParam->MBS_Data_Request.Flags.IsRetry = 0;  
		pMailBoxParam->MBS_Data_Request.Destination.Val = ReplyAddress.Val;
		MailBoxState = MBS_Data_Request;
		continue;
	}

	// if I got this far and I'm sleepy, joined, PresenceAnnounced, and not busy, I can start sleeping
#if defined(DARC_SLEEPY_END)
	if (!DARCFlags.Bits.StayAwakeToStream && DARCFlags.Bits.PresenceAnnounced) 	
		MailBoxState = MBS_Device_Sleep_Request;
#endif
		
   }  // end of while (1) loop
}


// Command driven I/O routines
void SetIOVal( BYTE IOval )
{
	pTxData[0] = DARC_STATUS_SUCCESS;
	pMailBoxParam->MBS_Data_Request.DataPayloadLen = 0x01;
	switch (IOval)
	{
		case IO_RA0:  
			if (NumANchannels>0)  goto IO_Failure;
			TRISAbits.TRISA0 = 0;	
			PORTAbits.RA0 = ( MessageCache[2] ? 1 : 0 ); 
			break;
		case IO_RA1:  
			if (NumANchannels>1)  goto IO_Failure;
			TRISAbits.TRISA1 = 0;	
			PORTAbits.RA1 = ( MessageCache[2] ? 1 : 0 ); 
			break;
		case IO_RA2:  
			if (NumANchannels>2)  goto IO_Failure;
			TRISAbits.TRISA2 = 0;	
			PORTAbits.RA2 = ( MessageCache[2] ? 1 : 0 ); 
			break;
		case IO_RA3:  
			if (NumANchannels>3)  goto IO_Failure;
			TRISAbits.TRISA3 = 0;	
			PORTAbits.RA3 = ( MessageCache[2] ? 1 : 0 ); 
			break;
		case IO_RB4:  
			TRISBbits.TRISB4 = 0;	
			PORTBbits.RB4 = ( MessageCache[2] ? 1 : 0 ); 
			break;
		case IO_RB5:  
			TRISBbits.TRISB5 = 0;	
			PORTBbits.RB5 = ( MessageCache[2] ? 1 : 0 ); 
			break;
		case IO_RB6:  
			TRISBbits.TRISB6 = 0;	
			PORTBbits.RB6 = ( MessageCache[2] ? 1 : 0 ); 
			break;
		case IO_RC6:  
			TRISCbits.TRISC6 = 0;	
			PORTCbits.RC6 = ( MessageCache[2] ? 1 : 0 ); 
			break;
		case IO_RC7:  
			TRISCbits.TRISC7 = 0;	
			PORTCbits.RC7 = ( MessageCache[2] ? 1 : 0 ); 
			break;
#ifdef PIXIE
		case IO_RE0:  
			if (NumANchannels>5)  goto IO_Failure;
			TRISEbits.TRISE0 = 0;	
			PORTEbits.RE0 = ( MessageCache[2] ? 1 : 0 ); 
			break;
		case IO_RE1:  
			if (NumANchannels>6)  goto IO_Failure;
			TRISEbits.TRISE1 = 0;	
			PORTEbits.RE1 = ( MessageCache[2] ? 1 : 0 ); 
			break;
		case IO_RE2:  
			if (NumANchannels>7)  goto IO_Failure;
			TRISEbits.TRISE2 = 0;	
			PORTEbits.RE2 = ( MessageCache[2] ? 1 : 0 ); 
			break;
#endif
		default:
IO_Failure:
			pTxData[0] = DARC_STATUS_IO_NOT_POSSIBLE;
	}
}

void GetIOVal( BYTE IOval )
{
	pTxData[0] = DARC_STATUS_SUCCESS;
	pMailBoxParam->MBS_Data_Request.DataPayloadLen = 0x02;

	switch (IOval)
	{
		case IO_RA0:  
			if (NumANchannels>0)  goto IO_Failure;
			TRISAbits.TRISA0 = 1;	
			pTxData[1] = PORTAbits.RA0 ;
			break;
		case IO_RA1:  
			if (NumANchannels>1)  goto IO_Failure;
			TRISAbits.TRISA1 = 1;	
			pTxData[1] = PORTAbits.RA1 ;
			break;
		case IO_RA2:  
			if (NumANchannels>2)  goto IO_Failure;
			TRISAbits.TRISA2 = 1;	
			pTxData[1] = PORTAbits.RA2 ;
			break;
		case IO_RA3:  
			if (NumANchannels>3)  goto IO_Failure;
			TRISAbits.TRISA3 = 1;	
			pTxData[1] = PORTAbits.RA3 ;
			break;
		case IO_RB4:  
			TRISBbits.TRISB4 = 1;	
			pTxData[1] = PORTBbits.RB4 ;
			break;
		case IO_RB5:  
			TRISBbits.TRISB5 = 1;	
			pTxData[1] = PORTBbits.RB5 ;
			break;
		case IO_RB6:  
			TRISBbits.TRISB6 = 1;	
			pTxData[1] = PORTBbits.RB6 ;
			break;
		case IO_RC6:  
			TRISCbits.TRISC6 = 1;	
			pTxData[1] = PORTCbits.RC6 ;
			break;
		case IO_RC7:  
			TRISCbits.TRISC7 = 1;	
			pTxData[1] = PORTCbits.RC7 ;
			break;
#ifdef PIXIE
		case IO_RE0:  
			if (NumANchannels>5)  goto IO_Failure;
			TRISEbits.TRISE0 = 1;	
			pTxData[1] = PORTEbits.RE0 ;
			break;
		case IO_RE1:  
			if (NumANchannels>6)  goto IO_Failure;
			TRISEbits.TRISE1 = 1;	
			pTxData[1] = PORTEbits.RE1 ;
			break;
		case IO_RE2:  
			if (NumANchannels>7)  goto IO_Failure;
			TRISEbits.TRISE2 = 1;	
			pTxData[1] = PORTEbits.RE2 ;
			break;
#endif
		default:
IO_Failure:
			pTxData[0] = DARC_STATUS_IO_NOT_POSSIBLE;
			pMailBoxParam->MBS_Data_Request.DataPayloadLen = 0x01;
	}
}

void GetANVal( BYTE IOval )
{
	pTxData[0] = DARC_STATUS_SUCCESS;
	pMailBoxParam->MBS_Data_Request.DataPayloadLen = 0x03;
	ADCON0 = 0b00000001;	// a/d on
	ADCON2 = 0b10000110;  	// a/d conv period 64Tosc, right shift result 
	switch (IOval)
	{
		case IO_AN0: 
			if (NumANchannels<=0)  goto IO_Failure;
			ADCON0 = 0b00000011;
			break;
		case IO_AN1: 
			if (NumANchannels<=1)  goto IO_Failure;
			ADCON0 = 0b00000111;
			break;
		case IO_AN2: 
			if (NumANchannels<=2)  goto IO_Failure;
			ADCON0 = 0b00001011;
			break;
		case IO_AN3: 
			if (NumANchannels<=3)  goto IO_Failure;
			ADCON0 = 0b00001111;
			break;
#ifdef PIXIE
		case IO_AN5: 
			if (NumANchannels<=5)  goto IO_Failure;
			ADCON0 = 0b00010111;
			break;
		case IO_AN6: 
			if (NumANchannels<=6)  goto IO_Failure;
			ADCON0 = 0b00011011;
			break;
		case IO_AN7: 
			if (NumANchannels<=7)  goto IO_Failure;
			ADCON0 = 0b00011111;
			break;
#endif
		default:
IO_Failure:
			pTxData[0] = DARC_STATUS_IO_NOT_POSSIBLE;
			pMailBoxParam->MBS_Data_Request.DataPayloadLen = 0x01;
			return;
	}
	
	// wait for I/O to complete
	while (ADCON0bits.NOT_DONE) {ClrWdt();}
	
	// record result, little endian
	pTxData[1] = ADRESL;
	pTxData[2] = ADRESH;

	return;
}

// set up number of A to D channels
void SetANChannels( void )
{
#ifdef PIXIE
	if ( MessageCache[1] > 8 )
#else
	if ( MessageCache[1] > 4 )
#endif
	{
		pTxData[0] = DARC_STATUS_OUT_OF_RANGE;
	}
	else
	{
		NumANchannels = MessageCache[1];
		ADCON1 = 0x0F - NumANchannels;
		pTxData[0] = DARC_STATUS_SUCCESS;
	}
}

void SetStream( void )
{
	pMailBoxParam->MBS_Data_Request.DataPayloadLen = 0x01;

	if (MessageLen!=0x03)
	{
		pTxData[0] = DARC_SYNTAX_ERROR;
		return;
	}
	
	StreamRate = MessageCache[1];
	StreamChannel = MessageCache[2];
	pTxData[0] = DARC_STATUS_SUCCESS;
	
#ifdef DARC_SLEEPY_END
	StreamCountDown = StreamRate;
#else
	LastStream = TickGet();
#endif
	
}




/*******************************************************************************
User Interrupt Handlers

The stack uses some interrupts at low priority for its internal processing.  Once it is done
checking for its interrupts, the stack calls UserInterruptHandler function to allow for any
additional interrupt processing.  Interrupts should not be disabled unless the sleep state 
has been entered.  

PriorityUserInterruptHandler must be for very fast, defined duration operations only

*******************************************************************************/


#pragma interrupt PriorityUserInterruptHandler
void PriorityUserInterruptHandler(void)
{
	// Only very fast functions here
}


void UserInterruptHandler(void)
{
	// Slower functions here
} 


/*******************************************************************************
ZigBeeHook

This function allows a sneak look at the state of the ZigBee stack.
If you care to investigate the ZigBee stack source, you can declare variables as extern
and look at them.  DO NOT modify them - that would invalidate stack compliance

********************************************************************************/

void ZigBeeHook( void )
{
}


/*******************************************************************************
PutROMString

Writes to the serial TxD output

********************************************************************************/


void PutROMString(ROM char* str)
{
    BYTE c;
    while( c = *str++ )  
    {
	    while( TXSTAbits.TRMT==0 );
	    TXREG = c ;
	}
}

/*******************************************************************************
NetworkFailed

Works out what to do if cannot be sure we're still connected to the network
This implementation just waits a second, then resets.  C`onsideration should
be given to whether a longer delay should be implemented for power saving

********************************************************************************/

void NetworkFailed( void )
{
	Sleep();
	Nop();
	Reset();
}

